from .main import SOARException
from .validation_exc import (
    DataError,
    NoInputError,
    NoSecretsError,
    BadInputError,
    BadSecretsError
)

from .protocol_exc import (
    ProtocolError,
    ConnectionFailed,
    CredentialsError,
    PermissionsError,
    ExecutionError
)